use crate::util::ToLitStr;
use syn::{spanned::Spanned as _, DataEnum, DataStruct, DataUnion, Fields, LitStr, Type};

#[allow(dead_code)]
pub(super) enum ParseData {
	Struct(Vec<(LitStr, Type)>),
	Enum(Vec<LitStr>),
	Alternatives(Vec<ParseData>),
	Unit
}

pub(super) fn parse_struct(strukt: &DataStruct) -> syn::Result<ParseData> {
	match &strukt.fields {
		Fields::Named(named_fields) => {
			let mut fields: Vec<(LitStr, Type)> = Vec::new();
			for f in &named_fields.named {
				let ident = f.ident.as_ref().ok_or_else(|| {
					syn::Error::new(f.span(), "#[derive(OpenapiType)] does not support fields without an ident")
				})?;
				let name = ident.to_lit_str();
				let ty = f.ty.to_owned();
				fields.push((name, ty));
			}
			Ok(ParseData::Struct(fields))
		},
		Fields::Unnamed(_) => unimplemented!(),
		Fields::Unit => Ok(ParseData::Unit)
	}
}

pub(super) fn parse_enum(_inum: &DataEnum) -> syn::Result<ParseData> {
	unimplemented!()
}

pub(super) fn parse_union(union: &DataUnion) -> syn::Result<ParseData> {
	Err(syn::Error::new(
		union.union_token.span(),
		"#[derive(OpenapiType)] cannot be used on unions"
	))
}
